import java.io.DataOutputStream;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;

/**
 * Created by gaetan on 28.01.15.
 */
public class TCPServer implements Runnable, WorkerRunnable.OnMessageReceived, WorkerRunnable.OnErrorReadingData, WorkerRunnable.OnClientDisconnect{

    protected int mServerPort = 4321;
    protected ServerSocket mServerSocket = null;
    protected boolean mIsStopped = false;
    protected Thread mRunningThread = null;
    private static final int MAX_MESSAGE_SIZE = 12000;
    protected ArrayList<Socket> mListClients = new ArrayList<Socket>();

    public TCPServer(int port){
        this.mServerPort = port;
    }

    @Override
    public void run() {
        synchronized(this){
            mRunningThread = Thread.currentThread();
        }

        openServerSocket();

        while (!isStopped()){
            Socket clientSocket = null;
            try {
                clientSocket = mServerSocket.accept();
            } catch (IOException e) {
                if(isStopped()) {
                    System.out.println("Server Stopped. " + e.toString()) ;
                    return;
                }
                throw new RuntimeException(
                        "Error accepting client connection", e);
            }

            mListClients.add(clientSocket);
            System.out.println("New client connected");

            new Thread(new WorkerRunnable(clientSocket,this)).start();
        }
        System.out.println("Server Stopped.") ;
    }


    private synchronized boolean isStopped() {
        return this.mIsStopped;
    }

    public synchronized void stop(){
        mIsStopped = true;
        try {
            mServerSocket.close();
        } catch (IOException e) {
            throw new RuntimeException("Error closing server", e);
        }
    }

    private void openServerSocket() {
        try {
            mServerSocket = new ServerSocket(this.mServerPort);
        } catch (IOException e) {
            throw new RuntimeException("Cannot open port " + this.mServerPort, e);
        }
    }

    public Boolean sendMessageToAll(byte[] bufferToSend) {
        if (mListClients.size()==0 || bufferToSend.length > MAX_MESSAGE_SIZE) {
            System.out.println("sendMessageToAll: message probably too long: " + bufferToSend.length +"(max is: "+MAX_MESSAGE_SIZE+")");
            return false;
        }

        try {
            long size = bufferToSend.length;

            DataOutputStream output;

            for (Socket s : mListClients) {
                output = new DataOutputStream(s.getOutputStream());
                output.writeLong(size);
                output.write(bufferToSend);
                output.flush();
            }

        } catch (IOException e) {
            System.out.println("An error occurred while sending the message: " + e.toString());
            e.printStackTrace();
            return false;
        } catch (Exception e) {
            System.out.println("An error occurred while sending the message: " + e.toString());
            e.printStackTrace();
            return false;
        }

        return true;
    }

    @Override
    public void onMessageReceived(String message) {
        System.out.println("Received message " + message);

        // append a null character to the string so that c++ client/server handle it right
        StringBuilder sb = new StringBuilder();
        sb.append(message);
        char nul = '\u0000';
        sb.append(nul);
        String s = sb.toString();

        if ("disconnect".equals(message)) {
            return;
        }
        if (sendMessageToAll(s.getBytes())) {
            System.out.println("Message successfully forwarded to every client");
        }
        else {
            System.out.println("Message hasn't been forwarded to every client");
        }
    }

    @Override
    public void onErrorReadingData(Socket client) {
        mListClients.remove(client);
        System.out.println("A client disconnected, it has been removed");
    }

    @Override
    public void onClientDisconnect(Socket client) {
        mListClients.remove(client);
        System.out.println("A client disconnected, it has been removed");
    }
}
