import sun.rmi.runtime.Log;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.Socket;

/**
 * Created by gaetan on 28.01.15.
 */
public class WorkerRunnable implements Runnable{

    protected Socket mClientSocket = null;
    protected DataInputStream mInput;
    protected DataOutputStream mOutput;
    protected Boolean mKeepRunning;

    protected OnMessageReceived mOnMessageReceivedListener;
    protected OnErrorReadingData mOnErrorReadingDataListener;
    protected OnClientDisconnect mOnClientDisconnectListener;

    private static final int MAX_MESSAGE_SIZE = 12000;

    private byte[] mBufferMsg = new byte[MAX_MESSAGE_SIZE];
    private byte[] mReceivedMessageSize = new byte[4];

    public WorkerRunnable(Socket mClientSocket, TCPServer listener) {
        this.mClientSocket = mClientSocket;
        mOnMessageReceivedListener = listener;
        mOnErrorReadingDataListener = listener;
        mOnClientDisconnectListener = listener;
    }

    public void run() {
        try {
            mInput = new DataInputStream(mClientSocket.getInputStream());
            mOutput = new DataOutputStream(mClientSocket.getOutputStream());

            mKeepRunning = true;
            while (mKeepRunning) {
                readIncomingMessages();
            }
           closeConnection();

        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void readIncomingMessages() {
        int bytesRead;
        try {
            bytesRead = mInput.read(mReceivedMessageSize, 0, 4);

            int size = ((mReceivedMessageSize[0] & 0xFF) << 24) | ((mReceivedMessageSize[1] & 0xFF) << 16) |
                    ((mReceivedMessageSize[2] & 0xFF) << 8) | (mReceivedMessageSize[3] & 0xFF);

            if (bytesRead == 4 && size > 0 && size <= MAX_MESSAGE_SIZE)  {
                int total_bytes_read = 0;
                while (total_bytes_read < size) {
                    bytesRead = mInput.read(mBufferMsg, total_bytes_read, size - total_bytes_read);
                    total_bytes_read += bytesRead;
                }
                if (total_bytes_read == size) {
                    String sMessageReceived = new String(mBufferMsg, 0, size);
                    sMessageReceived = filterMessage(sMessageReceived);
                    mOnMessageReceivedListener.onMessageReceived(sMessageReceived);
                }
                else {
                    mOnErrorReadingDataListener.onErrorReadingData(mClientSocket);
                    mKeepRunning = false;
                }
            }
            else if (bytesRead == -1) {
                System.out.println("Byte read = -1, closing connection with socket");
                stopConnection();
            }
        } catch (IOException e) {
            System.out.println("In IOocception, closing connection with socket");
            stopConnection();
        }
    }

    private String filterMessage(String message) {
        message = message.replaceAll("\\x00", "");

        if ("disconnect".equals(message)) {
            mOnClientDisconnectListener.onClientDisconnect(mClientSocket);
        }

        return message;
    }

    private void stopConnection() {
        mOnErrorReadingDataListener.onErrorReadingData(mClientSocket);
        mKeepRunning = false;
    }

    private void closeConnection()  {
        try {
            mOutput.close();
            mInput.close();
            mClientSocket.close();
            System.out.println("Client' socket closed");
        } catch (IOException e1) {
            System.out.println("Error closing client' socket " + e1.getMessage());
        }
    }
    public interface OnMessageReceived {
        public void onMessageReceived(String message);
    }

    public interface OnErrorReadingData {
        public void onErrorReadingData(Socket client);
    }

    public interface OnClientDisconnect {
        public void onClientDisconnect(Socket client);
    }
}
